/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006  Christofer Wesseling, Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#include "umtsmon_global.h"
#include "Profile.h"
#include <assert.h>

// to prevent keys from varying between setter and getter
// the below static const strings are defined.
static const char* theProfileDefaultStr = "/profiles/Default";
static const char* theDefaultRouteStr = "DefaultRoute";
static const char* theApnStr = "apn";
static const char* theUsernameStr = "username";
static const char* thePasswordStr = "password";
static const char* theNoIpDefaultStr = "noIpDefault";
static const char* theReplaceDefaultRouteStr = "replaceDefaultRoute";
static const char* theUsePeerDnsStr = "usePeerDns";
static const char* theNoCompressionStr = "noCompression";
static const char* theNoAuthStr = "noAuth";



//constructor 1
Profile::Profile(const QString & aProfileName)
{
	DEBUG2("Profile::Profile('%s')\n", aProfileName.ascii());
	if (theProfileName.isEmpty())
	{
		theProfileName = "Default";
	} 	
	theProfileName = aProfileName;
}

//constructor 2
Profile::Profile(void)
{
	DEBUG2("Profile::Profile(---)\n");
	// this is the 'default' constructor, that takes the default profile
	// if there is no profile yet, we create a 'Default' profile.
	theProfileName = TheSettingsSingleton::getQSRef().readEntry(theProfileDefaultStr, "");
	if (theProfileName.isEmpty())
	{
		theProfileName = "Default";
	} 	
}

////////////////////////////////////////////////////////////
// GETTERS: return the .... setting of the current profile

 
QString
Profile::getAPN(void) const
{	
	return TheSettingsSingleton::getQSRef().readEntry(getKeyPathName(theApnStr), "");
}

QString
Profile::getUsername(void) const
{
	return TheSettingsSingleton::getQSRef().readEntry(getKeyPathName(theUsernameStr), "");
}

QString 
Profile::getPassword(void) const
{
	return TheSettingsSingleton::getQSRef().readEntry(getKeyPathName(theUsernameStr), "");
}

bool
Profile::getNoIpDefault(void) const
{
	return TheSettingsSingleton::getQSRef().readBoolEntry(getKeyPathName(theNoIpDefaultStr), false);
}

bool 
Profile::getIsDefaultRoute(void) const
{
	return TheSettingsSingleton::getQSRef().readBoolEntry(getKeyPathName(theDefaultRouteStr), true);
}

bool 
Profile::getReplaceDefaultRoute(void) const
{
	return TheSettingsSingleton::getQSRef().readBoolEntry(getKeyPathName(theReplaceDefaultRouteStr), true);
}

bool 
Profile::getUsePeerDns(void) const
{
	return TheSettingsSingleton::getQSRef().readBoolEntry(getKeyPathName(theUsePeerDnsStr), true);
}

bool 
Profile::getNoCompression(void) const
{
	return TheSettingsSingleton::getQSRef().readBoolEntry(getKeyPathName(theNoCompressionStr), false);
}

bool 
Profile::getNoAuth(void) const
{
	return TheSettingsSingleton::getQSRef().readBoolEntry(getKeyPathName(theNoAuthStr), false);
}


////////////////////////////////////////////////////////////
// SETTERS: change the .... setting of the current profile

bool 
Profile::setApn(QString anApn)
{
	return TheSettingsSingleton::getQSRef().writeEntry(getKeyPathName(theApnStr), anApn );
}

bool 
Profile::setUsername(QString anUsername)
{
	return TheSettingsSingleton::getQSRef().writeEntry(getKeyPathName(theUsernameStr), anUsername );	
}

bool Profile::setPassword(QString aPassword)
{
	return TheSettingsSingleton::getQSRef().writeEntry(getKeyPathName(thePasswordStr), aPassword );
}

bool 
Profile::setNoIpDefault(bool aNoIpDefault)
{
	return TheSettingsSingleton::getQSRef().writeEntry(getKeyPathName(theNoIpDefaultStr), aNoIpDefault);
}

bool 
Profile::setIsDefaultRoute(bool aDefaultRoute)
{
	return TheSettingsSingleton::getQSRef().writeEntry(getKeyPathName(theDefaultRouteStr), aDefaultRoute );
}

bool 
Profile::setReplaceDefaultRoute(bool aReplaceDefaultRoute)
{
	return TheSettingsSingleton::getQSRef().writeEntry(getKeyPathName(theReplaceDefaultRouteStr), aReplaceDefaultRoute );
}

bool 
Profile::setUsePeerDns(bool anUsePeerDns)
{
	return TheSettingsSingleton::getQSRef().writeEntry(getKeyPathName(theUsePeerDnsStr), anUsePeerDns );
}

bool 
Profile::setNoCompression(bool aNoCompression)
{
	return TheSettingsSingleton::getQSRef().writeEntry(getKeyPathName(theNoCompressionStr), aNoCompression );
}

bool 
Profile::setNoAuth(bool aNoAuth)
{
	return TheSettingsSingleton::getQSRef().writeEntry(getKeyPathName(theNoAuthStr), aNoAuth );
}

////////////////////////////////////////////////////////////
// save the profile to the HDD - New APN Name = New Profile
void 
Profile::save(void) const
{
	TheSettingsSingleton::makeChangesPersistent();
}

////////////////////////////////////////////////////////////
bool 
Profile::remove(void) const
{
	DEBUG1("Request to remove profile '%s'\n", theProfileName.ascii());
	return TheSettingsSingleton::removeSubTree("/profiles/" + theProfileName);
}

////////////////////////////////////////////////////////////
bool 
Profile::setAsDefault(void) const
{
	return TheSettingsSingleton::getQSRef().writeEntry(theProfileDefaultStr, theProfileName);
}
