/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2007  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef THEDEVICEMANAGERSINGLETON_CLASS
#define THEDEVICEMANAGERSINGLETON_CLASS

#include "AutoDetect.h"
#include "DeviceCapabilities.h"


//////////////////////////////////////////////////////////////////////////////

/** TheDeviceManagerSingleton abstracts, checks and protects the active instance
 *  of the Device class
 */
class TheDeviceManagerSingleton
{
private:
	/** private constructor as demanded by the Singleton Design Pattern
	 *  will be called by the get###() method if needed
	 */
	TheDeviceManagerSingleton(void);
	/** private destructor as demanded by the Singleton Design Pattern
	 */
	~TheDeviceManagerSingleton(void);	
	
	/// private, unimplemented copy constructor - don't use
	TheDeviceManagerSingleton(const TheDeviceManagerSingleton& aPtr);
	/// private, unimplemented assign operator - don't use
	TheDeviceManagerSingleton& operator=(TheDeviceManagerSingleton& aPtr);

public:
	/// 
	static DeviceCapabilities& getDeviceCapsRef(void);
	static DeviceRadioCapabilities& getDeviceRadioCapsRef(void);
	static ConnectionInfo* getConnectionInfoPtr(void);
	
	/// the main entry member to the Singleton
	static TheDeviceManagerSingleton& me(void);
	
	/// used by main() to set a command-line specified AT port name
	void setSuggestedAT(const QString& anATPortName);
	/// used by main() to set a command-line specified PPP port name
	void setSuggestedPPP(const QString& aPPPPortName);
	/// used by main() to set the device type	
	void setSuggestedDeviceType(DeviceCapabilities::CardType  aDeviceType);
	/// used by main() to force autodetection, instead of relying on config file
	void forceAutodetection(void);

private:
	bool setupDevice(void);
	void readFromConfigFile(void);
	void writeToConfigFile(void);
	bool doDeviceCreation(AutoDetectBase& aSuggestedAutoDetector);

	// typedef to simplify usage of the AutoDetect list
	typedef std::vector<AutoDetectBase*> AutoDetectList;
	
	/// list of all various AutoDetect subclass instances
	AutoDetectList theAutoDetectList;
	
	/** specialised autodetect pointer, to the autodetect class that can 
	 *  actually build a Device from supplied parameters.
	 *  it's also the first entry of the AutoDetectList.
	 */
	NonAutoDetect* theNonAutoDetectPtr;
};



#endif // THEDEVICEMANAGERSINGLETON_CLASS
