/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006, 2007  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "umtsmon_global.h"

#include "PPPConnection.h"
#include "Query.h"
//#include "Device.h"
#include "Runner.h"
#include "Profile.h"
#include "TheDeviceManagerSingleton.h"
#include "TheSettingsSingleton.h"
#include "Popup.h"

#include <qfile.h>
#include <qregexp.h>

#include <qstring.h>
#include <sys/types.h>
#include <signal.h>

// the actual definition of thePPPDState
static PPPObserverInterface::PPPDState  thePPPDState = PPPObserverInterface::STOPPED;
static pid_t theRunningPPPDPID = -1;


// the function to be called by atexit 
// to make sure the PPPD daemon is shut down
void run_at_exit(void)
{
	DEBUG5("PPPConnection run_at_exit() called\n");
	Runner myRunner;
	int myPID = myRunner.doesPPPDexist(); 
	if (myPID!=-1)
	{
		if (thePPPDState != PPPObserverInterface::NOT_OURS_RUNNING)
		{
			// thePPPDState is either RUNNING or STARTING
			DEBUG3("Killing PPPD - process %d\n", myPID);
			kill(myPID, SIGTERM);
			theRunningPPPDPID = -1;
		}
	}
}



PPPConnection::PPPConnection(void) :
			theCIInhibitorPtr(NULL),
			hasOnlyOnePort(false),
	 		theCounterBytesSent(0),
			theCounterBytesReceived(0),
			thePPPThreadPtr(NULL)
{
	atexit(run_at_exit);
	
	theSerialPortName = TheDeviceManagerSingleton::getDeviceCapsRef().getPPPSerialPortName();
	assert(!theSerialPortName.isEmpty());
	
	// check if AT and PPP are the same device
	if (TheDeviceManagerSingleton::getDeviceCapsRef().getATSerialPortName() == theSerialPortName)
		hasOnlyOnePort = true;
	
	// let's assume the worst
	setNewPPPState(PPPObserverInterface::NOT_OURS_RUNNING);
	isPPPConnected();
} 


PPPConnection::~PPPConnection(void) 
{ 
	DEBUG4("PPPConnection::~PPPConnection(void)\n");
	stopPPP(); 
}


void 
PPPConnection::stopPPP(void)
{
	run_at_exit();
}

/*
void 
PPPConnection::setDeviceInfo(const DeviceCapabilities* aDeviceCapabilitiesPtr)
{
	theSerialPortName = aDeviceCapabilitiesPtr->getPPPSerialPortName();
	assert(!theSerialPortName.isEmpty());
	
	// check if AT and PPP are the same device
	if (aDeviceCapabilitiesPtr->getATSerialPortName() == theSerialPortName)
		hasOnlyOnePort = true;
	
	// let's assume the worst
	setNewPPPState(PPPObserverInterface::NOT_OURS_RUNNING);
	isPPPConnected();
}
*/

bool PPPConnection::isPPPStarting(void)
{
	// if PPP is running, it is not starting anymore :-)
	if (isPPPConnected())
		return false;
	return thePPPDState == PPPObserverInterface::STARTING;
}


bool 
PPPConnection::isPPPConnected(void)
{
	// FIXME: unfortunately, this is a rather expensive operation...
	// this can be worked around by checking if we know where to look first
	pid_t myPID = Runner::doesPPPDexist();
	
	switch (thePPPDState)
	{
	case PPPObserverInterface::ERROR:
		setNewPPPState(PPPObserverInterface::STOPPED);
		break;
	case PPPObserverInterface::STOPPED:
		if (myPID!=-1)
		{
			DEBUG2("PPPConnection::isPPPConnected someone else started a pppd\n");
			setNewPPPState(PPPObserverInterface::NOT_OURS_RUNNING);
		}
		if (theRunningPPPDPID!=-1)
		{
			// shouldn't happen
			DEBUG1("PPPConnection::isPPPConnected state changed to STOPPED "
			       "without updating theRunningPPPDPID\n");
			assert(false);
			theRunningPPPDPID=-1;
		}
		break;
	case PPPObserverInterface::STARTING:
		if (thePPPThreadPtr == NULL)
		{
			// something went wrong ?!?
			setNewPPPState(PPPObserverInterface::STOPPED);
			thePPPDStdErr.push_back(APPNAME " internal Error: not thePPPThreadPtr");
			break; 			
		}
		if (thePPPThreadPtr->isDone())
		{
			DEBUG2("isDone is true\n");
			// so, was it successful or not?
			if (myPID == theRunningPPPDPID)
			{
				// euh? finished *and* still running?
				// that means we have a zombie 
				//    - process is dead, we have to reap it...
				myPID = -1; 
			}
			if (myPID == -1)
			{
				theRunningPPPDPID=-1;
				thePPPDStdOut = thePPPThreadPtr->getStdOut();
				thePPPDStdErr = thePPPThreadPtr->getStdErr();
				if(theVerbosity>4)
				{
					QStringList::Iterator it;
					DEBUG4("*****output of PPP to stdout\n");
					for (it = thePPPDStdOut.begin(); it != thePPPDStdOut.end(); ++it )
					{
				        DEBUG4("* %s\n", (*it).ascii());
					}
					DEBUG4("*****end-of-output\n");
					DEBUG4("*****output of PPP to stderr\n");
					for (it = thePPPDStdErr.begin(); it != thePPPDStdErr.end(); ++it )
					{
				        DEBUG4("* %s\n", (*it).ascii());
					}
					DEBUG4("*****end-of-output\n");
				}
				setNewPPPState(PPPObserverInterface::ERROR);
			}
			else
			{
				theRunningPPPDPID = myPID;
				setNewPPPState(PPPObserverInterface::RUNNING);
			}
			DEBUG3("thread end - wait and cleanup\n");
			delete thePPPThreadPtr;
			thePPPThreadPtr = NULL;
		}
		else
		{
			DEBUG3("PPPDThread still running\n");
		}
		break;
	case PPPObserverInterface::RUNNING:
		if (myPID==-1 || myPID!=theRunningPPPDPID)
		{
			DEBUG2("Our PPPD was killed by someone\n");
			theRunningPPPDPID=-1;
			setNewPPPState(PPPObserverInterface::STOPPED);
		}
		break;
	case PPPObserverInterface::NOT_OURS_RUNNING:
		if (myPID==-1)
		{
			DEBUG2("Someone else's PPPD was killed\n");
			setNewPPPState(PPPObserverInterface::STOPPED);
		}
		break;
	}
	
	DEBUG5("The PPPDState is %d\n", thePPPDState);
	
	// we need the ConnectionInfoInhibitor here if we have only one serial port
	// and we're not in 'stopped'.
	// note that PPPThread and startPPP also have CIIs to make sure that we
	// don't run into simulatenous accessess.
	if (hasOnlyOnePort)
	{
		if (thePPPDState==PPPObserverInterface::STOPPED)
		{
			if (theCIInhibitorPtr!=NULL)
			{
				delete theCIInhibitorPtr;
				theCIInhibitorPtr=NULL;
			}
		}
		else
		{
			if (theCIInhibitorPtr==NULL)
			{
				theCIInhibitorPtr = new ConnectionInfoInhibitor;
			}
		}
	}
		
	if (thePPPDState==PPPObserverInterface::STOPPED 
			|| thePPPDState==PPPObserverInterface::STARTING)
		return false;
	else
		return true;
}


bool 
PPPConnection::startPPP(const Profile& aProfile)
{
	// just as a precaution, some sanity checks.
	// we will be checking if a PPP connection is already running. 
	// This shouldn't be the case - that's why there is an assert(false).
	// however, if this happens in a release build, just return false.
	DEBUG3("PPPConnection::startPPP entry\n");
	if (isPPPConnected())
	{
		// if happens in debug build: make a lot of noise
		// if happens in release build: just error out of here
		DEBUG2("PPPConnection::startPPP, PPP is already connected ?!?\n");
		stopPPP();
		sleep(1);
		// this shouldn't happen - crash big time on debug builds
		assert(isPPPConnected()==false);
		// and a fall-through prevention for release builds
		if (isPPPConnected())
			return false;
	}
	
	// remove everything that might be left from previous connections
	assert(thePPPThreadPtr==NULL);
	thePPPDStdOut.clear();
	thePPPDStdErr.clear();

	
	// from now on until the exit of this member, prevent 
	// ConnectionInfo from interfering with the AT serial port
	setNewPPPState(PPPObserverInterface::STARTING);
	assert(theCIInhibitorPtr == NULL);
	if (hasOnlyOnePort)
		theCIInhibitorPtr = new ConnectionInfoInhibitor();
	ConnectionInfoInhibitor myStartingInhibitor;

	// we cannot connect if this is not set
	if ( theSerialPortName .isEmpty())
	{
		DEBUG2("PPPConnection::startPPP - no PPP device ?!?\n");
		return false;
	}


	// Do the 'connect' magic :-)	
	DEBUG2("before the connect magic!!!\n");
	// we need the braces {} here to make sure that the Query class is 
	// destroyed immediately after usage 
	{
	// AT&F: reset modem to default profile
	// FIXME: if this fails, we should do pccardctl eject and insert and retry
	Query myQuery("AT&F");
	if (myQuery.runUntilDone() != Query::OK)
		return false;
	}
	{
	// ATE1: enable echo
	// FIXME: if this fails, we should do pccardctl eject and insert and retry
	Query myQuery("ATE1");
	if (myQuery.runUntilDone() != Query::OK)
		return false;
	}

	// build up the connectionstring 
	// like "AT+CGDCONT=1,\"IP\",\"office.vodafone.nl\""
	//
	// security check: anAPN should not contain any quotes or spaces
	QString myAPN = aProfile.getAPN();
	if (myAPN.contains("\"") || myAPN.contains("'") || myAPN.contains(" ") )
	{
		DEBUG1("ERROR: APN contains illegal character\n");
		return false;
	}
	// FIXME: myCID should be made flexible in the future, 
	// needs to be a different number for cell phones on USB or BT 
	const int myCID = 1;
	QString myConnectStr = "AT+CGDCONT=";
		    myConnectStr += myConnectStr.number(myCID);
		    myConnectStr += ",\"IP\",\"" + myAPN + "\"";
	{
	Query myQuery(myConnectStr);
	if (myQuery.runUntilDone() != Query::OK)
		return false;
	}
	{
	Query myQuery("ATD*99***" + myConnectStr.number(myCID) + "#");
	// don't expect an OK following this one...
	myQuery.run();
	}

	QStringList myPPPArguments = assembleArgumentList(aProfile);

	QStringList::const_iterator i = myPPPArguments.begin();
	DEBUG4("Assembled PPP string is: '");
	for (; i != myPPPArguments.end(); i++)
		DEBUG4NT("%s ", (*i).ascii());
	DEBUG4("'\n");

	setNewPPPState(PPPObserverInterface::STARTING);
	thePPPThreadPtr = new PPPThread(myPPPArguments);
	thePPPThreadPtr->start();
	// if there is a PPP running now, it's ours.
	theRunningPPPDPID = Runner::doesPPPDexist();
	return true;
}



QStringList PPPConnection::assembleArgumentList(const Profile& aProfile) const
{
	// FIXME: lookup the meaning of the remainders in this string and move it
  	QStringList myPPPArguments;
  	myPPPArguments.push_back("idle");   myPPPArguments.push_back("7200");
	if (aProfile.getNoAuth())
		myPPPArguments.push_back("noauth");
  	myPPPArguments.push_back("asyncmap"); myPPPArguments.push_back("0");

	// PPP related stuff
	// we only ask PPP to detach if connection is successfully up
	myPPPArguments.push_back("updetach");
	if (theVerbosity > 2)
	{
		myPPPArguments.push_back("dump");
		myPPPArguments.push_back("debug");
		myPPPArguments.push_back("debug");
		myPPPArguments.push_back("debug");
	}
	
	// modem-related stuff
	myPPPArguments.push_back("460800");
	myPPPArguments.push_back("lock"); 
	myPPPArguments.push_back("crtscts");
	myPPPArguments.push_back("modem"); 
	myPPPArguments.push_back(theSerialPortName);
	
	// header compressions and such
	myPPPArguments.push_back("noipx");
	if (aProfile.getNoCompression())
		myPPPArguments.push_back("novj");
	
	// routing related stuff
	if (aProfile.getIsDefaultRoute())
		myPPPArguments.push_back("defaultroute");
	if (aProfile.getReplaceDefaultRoute())
		myPPPArguments.push_back("replacedefaultroute");
	if (aProfile.getUsePeerDns())
		myPPPArguments.push_back("usepeerdns");
	if (aProfile.getNoIpDefault())
		myPPPArguments.push_back("noipdefault");
  		
	// login info
	if (!aProfile.getUsername().isEmpty())
	{
		myPPPArguments.push_back("user");
		myPPPArguments.push_back(aProfile.getUsername());
	}
	if (!aProfile.getPassword().isEmpty())
	{
		myPPPArguments.push_back("password"); 
		myPPPArguments.push_back(aProfile.getPassword());
	}
	
	return myPPPArguments;
}




void PPPConnection::refreshPPPStats(void)
{
	DEBUG3("PPPConnection::refreshPPPStats\n");
	long myCounterBytesSent = 0;
	long myCounterBytesReceived = 0;
 	
	// FIXME: insert the 'my' again
	if (isPPPConnected()==false)
	{
		theCounterBytesSent = 0;
		theCounterBytesReceived = 0;
		return;
	}

	// update the rx and tx byte views
	QFile myFile("/proc/net/dev");
	if (myFile.open(IO_ReadOnly) == true)
	{
		QString myLine;
		// FIXME: This is hardcoded for ppp0!!!
		QRegExp myPPP("ppp0");
		while(myFile.readLine(myLine, 255)> 0)
		{
			// perform substring searching
			if (myPPP.search(myLine,0) != -1)
			{
				QStringList myList = QStringList::split(QRegExp("[ :]"), myLine);
				myCounterBytesSent = myList[9].toLong();
				myCounterBytesReceived = myList[1].toLong();
				break;	// breaks while loop
			}
		}
		myFile.close();
	}
	
	// if 'my' and 'the' are not equal, apparently something has happened
	// three cases:
	//   1) more data sent 
	//   2) data sent dropped to zero - connection was killed?
	//   3) data sent increased from zero - connection was started?
	// in the latter two cases we might want to update the PPP state
	if ((myCounterBytesSent != theCounterBytesSent || 
	       myCounterBytesReceived != theCounterBytesReceived ) 
	    && 
		(myCounterBytesSent == 0 || theCounterBytesSent == 0))
	{
		isPPPConnected();
	}
	theCounterBytesSent = myCounterBytesSent;
	theCounterBytesReceived = myCounterBytesReceived;
}


void PPPConnection::setNewPPPState(PPPObserverInterface::PPPDState aNewState)
{
	// only go through the burden of notify if state really changed
	if (aNewState != thePPPDState)
	{
		DEBUG3("PPPConnection::setNewPPPState state %d to %d\n", thePPPDState, aNewState);
		thePPPDState = aNewState;
		PPPObserverInterface* myObserverPtr = firstObserver();
		while (myObserverPtr != NULL)
		{
			notifyObserver(myObserverPtr);
			myObserverPtr = nextObserver();
		}
	}
}

void PPPConnection::notifyObserver(PPPObserverInterface* myObserverPtr) const 
{
	myObserverPtr->newPPPState(thePPPDState);
}

PPPConnection::PPPThread::PPPThread(const QStringList& anArgumentList)
	: QThread(), theArgumentList(anArgumentList), theReturnValue(-1), isRunning(false)
{
	DEBUG2("PPPConnection::PPPThread::PPPThread()\n");
}


void PPPConnection::PPPThread::run()
{
	DEBUG2("PPPThread::run, before runCommand\n");
	theReturnValue = theRunner.runCommand(Runner::PPPD, theArgumentList);
	DEBUG2("PPPThread::run, after runCommand\n");
	// end of the thread - we're done
	isRunning = false;
}
