/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006 Christofer Wesseling
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include "umtsmon_global.h"
#include "FileStuff.h"
#include "ProgressDialog.h"
#include "Query.h"
#include <qfile.h>



void InfoDialogUmtsmon::init()
{
	// FIXME: the tables of modules&devices can't be sorted on other colums by the user BUG: 1616485
	//        it musn't be implemented at this position, it's just a reminder...
	DEBUG2("InfoDialogUmtsmon::init called\n");
	bool myReady = false;
	// create a Status Dialog....
	ProgressDialog myPD(this, "Fetching device informations..."); 
	myPD.start(3000);
	// wait up to 10 seconds
	myPD.setMaxTime(10);

	// FIXME: Klaas thinks this while with progress dialog does not work like it should...
	while ( (myPD.runOneSecond() && !myReady) )
	{
		DEBUG2("run one second.. try to fetch sim/device informtions\n");
		lbl_IMEI->setText( get_Info_using_AT_Query("AT+CGSN").left(15) );
		lbl_IMSI->setText(get_Info_using_AT_Query("AT+CIMI"));
		lbl_Manufactor->setText(get_Info_using_AT_Query("AT+CGMI"));
		lbl_CardType->setText(get_Info_using_AT_Query("AT+CGMM"));
		lbl_Revision->setText(get_Info_using_AT_Query("AT+CGMR").left(6));
		
		// have we got infos from device? - if so, ready!
		// test on 3 values, that seems to be enough...
		if( (lbl_IMSI->text() != "error") && lbl_Manufactor->text() != "error"  && (lbl_Revision->text() != "error") ) 
			myReady = true; 
	}
	// finish the progress dialog.. we're ready..
	myPD.finish();

	// read the system informations...
	lbl_System_Version->setText(FileStuff::getFileFirstLine("/proc/version"));
	readSystemInformations();
}


/**
* returns the result of the query
* @param the Query String
* @return the result of the query,  "error" if no result
**/
QString InfoDialogUmtsmon::get_Info_using_AT_Query( QString anAtCommand )
{		
	Query myQuery(anAtCommand);
	if (myQuery.runUntilDone() != Query::OK)
	{
		// no big deal, it happens, fail silently
		DEBUG2("Query.run() failed\n");	
		return "error";
	}
	return myQuery.getAnswer().first();
}


/// reads systeminformations into the info dialog
void InfoDialogUmtsmon::readSystemInformations()
{

// System modules
	QString myLine;
	QFile f;

	f.setName( "/proc/modules" );
	f.open( IO_ReadOnly);
	while(!f.atEnd ())
	{
		f.readLine(myLine,900); // how long will an entry max be..?
		myLine = myLine.stripWhiteSpace ();
		// insert into table..
		/// new row at line 0
		module_table->insertRows(0);
		//insert into row,col,value 
		module_table->setText(0,0, myLine.section(" ",0,0) );
		module_table->setText(0,1, myLine.section(" ",1,1) );
		module_table->setText(0,2, myLine.section(" ",2,2) );
	}
	f.close();
	module_table->sortColumn(0,true,true); // sort on module_names, whole row
	module_table->adjustColumn(0);
	module_table->adjustColumn(1);
	module_table->adjustColumn(2);

	
//System devices
	f.setName( "/proc/devices" );
	f.open( IO_ReadOnly);

	f.readLine(myLine,50); // first line is just "Character devices:"
	while(!f.atEnd ())
	{
		f.readLine(myLine,50); // how long will an entry max be..?
		myLine = myLine.stripWhiteSpace ();
		// insert into table..
		/// new row at line 0
		if( ( myLine.left(3).isEmpty() ) ) // we have reached the last device.. :)
			break; 
		
		device_table->insertRows(0);
		//insert into row,col,value 
		device_table->setText(0,0, myLine.section(" ",0,0) ); // up to digit 3
		device_table->setText(0,1, myLine.section(" ",1) ); //from digit 4 to end..
	}
	f.close();
	device_table->sortColumn(1,true,true); // sort on device-name, whole row
	device_table->adjustColumn(0);
	device_table->adjustColumn(1);
}
